class StatisticRecord:
    def __init__(self, spikes, mask_spikes, total_threshold):
        self.spikes = spikes
        self.mask_spikes = mask_spikes
        self.total_threshold = total_threshold
